@@ -226,13 +226,6 @@ de265_error seq_parameter_set::read(error_queue* errqueue, bitreader* br)
     separate_colour_plane_flag = 0;
   }
 
-  if (separate_colour_plane_flag) {
-    ChromaArrayType = 0;
-  }
-  else {
-    ChromaArrayType = chroma_format_idc;
-  }
-
   if (chroma_format_idc<0 ||
       chroma_format_idc>3) {
     errqueue->add_warning(DE265_WARNING_INVALID_CHROMA_FORMAT, false);
@@ -272,6 +265,11 @@ de265_error seq_parameter_set::read(error_queue* errqueue, bitreader* br)
 
   READ_VLC_OFFSET(bit_depth_luma,  uvlc, 8);
   READ_VLC_OFFSET(bit_depth_chroma,uvlc, 8);
+  if (bit_depth_luma > 16 ||
+      bit_depth_chroma > 16) {
+    errqueue->add_warning(DE265_WARNING_SPS_HEADER_INVALID, false);
+    return DE265_ERROR_CODED_PARAMETER_OUT_OF_RANGE;
+  }
 
   READ_VLC_OFFSET(log2_max_pic_order_cnt_lsb, uvlc, 4);
   MaxPicOrderCntLsb = 1<<(log2_max_pic_order_cnt_lsb);
@@ -458,13 +456,20 @@ de265_error seq_parameter_set::read(error_queue* errqueue, bitreader* br)
 }
 
 
-de265_error seq_parameter_set::compute_derived_values()
+de265_error seq_parameter_set::compute_derived_values(bool sanitize_values)
 {
   // --- compute derived values ---
 
   SubWidthC  = SubWidthC_tab [chroma_format_idc];
   SubHeightC = SubHeightC_tab[chroma_format_idc];
 
+  if (separate_colour_plane_flag) {
+    ChromaArrayType = 0;
+  }
+  else {
+    ChromaArrayType = chroma_format_idc;
+  }
+
   if (ChromaArrayType==0) {
     WinUnitX = 1;
     WinUnitY = 1;
@@ -506,8 +511,35 @@ de265_error seq_parameter_set::compute_derived_values()
   Log2MinTrafoSize = log2_min_transform_block_size;
   Log2MaxTrafoSize = log2_min_transform_block_size + log2_diff_max_min_transform_block_size;
 
-  if (max_transform_hierarchy_depth_inter > Log2CtbSizeY - Log2MinTrafoSize) { return DE265_ERROR_CODED_PARAMETER_OUT_OF_RANGE; }
-  if (max_transform_hierarchy_depth_intra > Log2CtbSizeY - Log2MinTrafoSize) { return DE265_ERROR_CODED_PARAMETER_OUT_OF_RANGE; }
+  if (max_transform_hierarchy_depth_inter > Log2CtbSizeY - Log2MinTrafoSize) {
+    if (sanitize_values) {
+      max_transform_hierarchy_depth_inter = Log2CtbSizeY - Log2MinTrafoSize;
+    } else {
+      fprintf(stderr,"SPS error: transform hierarchy depth (inter) > CTB size - min TB size\n");
+      return DE265_ERROR_CODED_PARAMETER_OUT_OF_RANGE;
+    }
+  }
+
+  if (max_transform_hierarchy_depth_intra > Log2CtbSizeY - Log2MinTrafoSize) {
+    if (sanitize_values) {
+      max_transform_hierarchy_depth_intra = Log2CtbSizeY - Log2MinTrafoSize;
+    } else {
+      fprintf(stderr,"SPS error: transform hierarchy depth (intra) > CTB size - min TB size\n");
+      return DE265_ERROR_CODED_PARAMETER_OUT_OF_RANGE;
+    }
+  }
+
+
+  if (sanitize_values) {
+    if (max_transform_hierarchy_depth_inter < Log2CtbSizeY - Log2MaxTrafoSize) {
+      max_transform_hierarchy_depth_inter = Log2CtbSizeY - Log2MaxTrafoSize;
+    }
+
+    if (max_transform_hierarchy_depth_intra < Log2CtbSizeY - Log2MaxTrafoSize) {
+      max_transform_hierarchy_depth_intra = Log2CtbSizeY - Log2MaxTrafoSize;
+    }
+  }
+
 
   Log2MinPUSize = Log2MinCbSizeY-1;
   PicWidthInMinPUs  = PicWidthInCtbsY  << (Log2CtbSizeY - Log2MinPUSize);
@@ -542,23 +574,28 @@ de265_error seq_parameter_set::compute_derived_values()
   if (pic_width_in_luma_samples  % MinCbSizeY != 0 ||
       pic_height_in_luma_samples % MinCbSizeY != 0) {
     // TODO: warn that image size is coded wrong in bitstream (must be multiple of MinCbSizeY)
+    fprintf(stderr,"SPS error: CB alignment\n");
     return DE265_ERROR_CODED_PARAMETER_OUT_OF_RANGE;
   }
 
   if (Log2MinTrafoSize > Log2MinCbSizeY) {
+    fprintf(stderr,"SPS error: TB > CB\n");
     return DE265_ERROR_CODED_PARAMETER_OUT_OF_RANGE;
   }
 
   if (Log2MaxTrafoSize > libde265_min(Log2CtbSizeY,5)) {
+    fprintf(stderr,"SPS error: TB_max > 32 or CTB\n");
     return DE265_ERROR_CODED_PARAMETER_OUT_OF_RANGE;
   }
 
 
   if (BitDepth_Y < 8 || BitDepth_Y > 16) {
+    fprintf(stderr,"SPS error: bitdepth Y not in [8;16]\n");
     return DE265_ERROR_CODED_PARAMETER_OUT_OF_RANGE;
   }
 
   if (BitDepth_C < 8 || BitDepth_C > 16) {
+    fprintf(stderr,"SPS error: bitdepth C not in [8;16]\n");
     return DE265_ERROR_CODED_PARAMETER_OUT_OF_RANGE;
   }
 
